package converter;

import java.io.File;
import java.util.InputMismatchException;
import java.util.Locale;
import java.util.Scanner;

public class Main {

	/**
	 * Legge il file Timing.GT.txt e produce Timing.GT.csv
	 * @param args
	 */
	public static void main(String[] args) {
		// check that arg exist
		if (args.length == 0)
		{
			System.out.println("time2csv v1.0");
			System.out.println("usage: java -jar time2csv.jar time-ground-truth-file.txt");
			return;
		}	
		// check that the file exists and is readable
		
		File myfile = new File(args[0]);
		if (!myfile.canRead())
		{
			System.out.println("cannot read " + myfile.getName());
			return;
		}
		
		/*
		 * Convert from text to CVS.
		 * 
		 * Input format:
		 * <video name>:<embedding type>:<time 1>:<time 2>[:<time 3>:<time 4>]
		 * If <embedding type> is 0, then no embedding has been performed. <time 1> and <time 2> denote the start and end times of the object of interest in the video, which are the same as the start and end times of the video.
		 * If <embedding type> is 1, then cut was used. <time 1> and <time 2> denote the start and end times of the object of interest in the video.
		 * If <embedding type> is 2, then dissolve was used. <time 1> and <time 2> denote the start and end times of the first dissolve (from unrelated content to the object of interest) and <time 3> and <time 4> denote the start and end times of the second dissolve (from the object of interest to unrelated content).
		 * example:
		 * EMB01_CAM/S0001_I00001_C001/EMB01_CAM_IMG_0947.MOV:2:28:29:32.63:34.63
		 * EMB01_CAM/S0001_I00001_C001/EMB01_CAM_IMG_0948.MOV:1:20:26.92
		 * 
		 * Ouput format:
		 * Use only the interval between time 2 and 3 for temporal localisation, i.e. where the object is fully visible.
		 * id, videoID, type, start, end
		 * <index>,<video name>,0,<time 1>,<time 2>
		 * <index>,<video name>,1,<time 1>,<time 2>
		 * <index>,<video name>,2,<time 2>,<time 3>
		 */
	     try {
	            Scanner scanner = new Scanner(myfile);
	            scanner.useLocale(Locale.US);
	        	
	        	System.out.println("id, videoID, type, start, end");
	        	
	        	int linecounter = 0;
	        	
	            while (scanner.hasNextLine()) {
	            	String line = scanner.nextLine();
	            	++ linecounter;
	            	String[] tokens = line.split(":");
	            	if (tokens.length == 4)
	            	{
            			System.out.format("%d,%s,%s,%s,%s %n", linecounter, tokens[0], tokens[1], tokens[2], tokens[3]);	            		
	            	}
	            	else if (tokens.length == 6)
	            	{
            			System.out.format("%d,%s,%s,%s,%s %n", linecounter, tokens[0], tokens[1], tokens[3], tokens[4]);	            			            		
	            	}
	            	
	            } // end while
	            scanner.close();	
	            
	        } catch ( InputMismatchException ime) {
	        	System.out.println("Input mismatch while reading text file: " + myfile.getName());
	        } catch (Exception ex) {
	            System.out.println("Text file " + myfile.getName() + " is missing.");
	        }
		}		
		

	

	}	// end MAIN
